﻿using System.Configuration;
using System.Data.SqlClient;
using NIntegrateQueryPerformanceComparison.LinqToSql;

namespace NIntegrateQueryPerformanceComparison.AdoNet
{
    class AdoNetCachedCommandTest : ITest
    {
        private static readonly string _connString = ConfigurationManager.ConnectionStrings[Constants.ConnectingStringName].ConnectionString;
        private static SqlCommand _cachedCommand;
        private static SqlCommand GetCachedCommand()
        {
            if (_cachedCommand == null)
            {
                _cachedCommand = new SqlCommand(
                    "SELECT TOP 1 ProductID,Name FROM SalesLT.Product WHERE ProductID = @ProductID")
                                     {CommandType = System.Data.CommandType.Text};
                _cachedCommand.Parameters.Add(new SqlParameter("@ProductID", 0));
            }

            return _cachedCommand.Clone();
        }

        private void DoExecute(int repeatTime)
        {
            for (var i = 1; i <= repeatTime; ++i)
            {
                using (var conn = new SqlConnection(_connString))
                {
                    if (conn.State != System.Data.ConnectionState.Open)
                        conn.Open();
                    var cmd = GetCachedCommand();
                    cmd.Parameters[0].Value = i;
                    cmd.Connection = conn;

                    using (var reader = cmd.ExecuteReader())
                    {
                        if (reader.Read())
                        {
                            var product = new Product
                                              {
                                                  ProductID = ((int) reader["ProductID"]),
                                                  Name = ((string) reader["Name"])
                                              };
                        }
                    }
                }
            }
        }

        #region ITest Members

        public void Prepare()
        {
            DoExecute(1);
        }

        public void Execute()
        {
            DoExecute(Constants.RepeatTime);
        }

        #endregion
    }
}
